"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TionApi = void 0;
const node_fetch_1 = __importDefault(require("node-fetch"));
const framework_1 = require("../homebridge/framework");
const platform_config_1 = require("../platform_config");
var AuthState;
(function (AuthState) {
    AuthState["NoToken"] = "no_token";
    AuthState["HasToken"] = "has_token";
    AuthState["TokenExpired"] = "token_expired";
})(AuthState || (AuthState = {}));
class TionApi {
    constructor(log, config, authApi) {
        this.log = log;
        this.config = config;
        this.authApi = authApi;
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.authApi.init();
        });
    }
    getSystemState() {
        return __awaiter(this, void 0, void 0, function* () {
            let firstRequest = false;
            if (!this.stateRequest) {
                this.log.debug('Loading system state');
                this.stateRequest = this._internalRequest('get', '/location', {
                    timeout: this.config.apiRequestTimeout,
                });
                firstRequest = true;
            }
            let stateResult;
            try {
                stateResult = yield this.stateRequest;
            }
            finally {
                if (firstRequest) {
                    this.stateRequest = undefined;
                }
            }
            let ret = null;
            if (this.config.homeName) {
                const lower = this.config.homeName.toLowerCase().trim();
                const location = stateResult.find(loc => loc.name.toLowerCase().trim() === lower);
                if (location) {
                    ret = location;
                }
                else {
                    this.log.warn(`Location ${this.config.homeName} not found, using first suitable`);
                }
            }
            if (!ret) {
                ret = stateResult.find(loc => loc.zones.length) || stateResult[0];
            }
            return ret;
        });
    }
    execCommand(deviceId, payload) {
        return __awaiter(this, void 0, void 0, function* () {
            this.log.debug(`TionApi.execCommand(deviceId = ${deviceId}, payload = ${JSON.stringify(payload)})`);
            try {
                let result = yield this._internalRequest('post', `/device/${deviceId}/mode`, {
                    body: payload,
                    timeout: this.config.apiRequestTimeout,
                });
                const commandId = result.task_id;
                this.log.debug(`TionApi.execCommand(commandId = ${commandId}, result = ${JSON.stringify(result)})`);
                let attempts = 0;
                while (result.status !== 'completed' && attempts++ < 4) {
                    switch (result.status) {
                        default:
                            this.log.error(`Unknown command status`);
                            this.log.error(result);
                            throw new Error('Status');
                            break;
                        case 'delivered':
                        case 'queued':
                            yield new Promise(resolve => setTimeout(resolve, attempts * 100));
                            result = yield this._internalRequest('get', `/task/${commandId}`, {
                                timeout: this.config.apiRequestTimeout,
                            });
                            this.log.debug(`TionApi.execCommand(result = ${JSON.stringify(result)})`);
                            break;
                    }
                }
                return result;
            }
            catch (err) {
                this.log.error('Failed to execute command');
                this.wrapError(err);
                this.log.error(err);
                throw err;
            }
        });
    }
    wrapError(err) {
        if (err.response) {
            err.response = undefined;
        }
    }
    _internalRequest(method, endpoint, options) {
        return __awaiter(this, void 0, void 0, function* () {
            let accessToken = this.authApi.getAccessToken();
            let state = accessToken ? AuthState.HasToken : AuthState.NoToken;
            let internalServerErrorAttempts = 0;
            while (true) {
                switch (state) {
                    case AuthState.HasToken:
                        this.log.debug('TionApi - has token');
                        try {
                            const headers = Object.assign({}, options.headers || {}, {
                                Authorization: `Bearer ${accessToken}`,
                            });
                            if (options.body !== undefined) {
                                headers['Content-Type'] = 'application/json';
                                options.body = JSON.stringify(options.body);
                            }
                            const result = yield node_fetch_1.default(`${TionApi.ApiBasePath}${endpoint}`, Object.assign(Object.assign({}, options), { headers, method: method.toUpperCase() }));
                            if (result.ok) {
                                return result.json();
                            }
                            if (result.status === 401) {
                                this.log.error('TionApi - token_expired: ', result.statusText);
                                state = AuthState.TokenExpired;
                            }
                            else if (result.status === 500 && internalServerErrorAttempts++ < 2) {
                                this.log.error(`TionApi - got internal server error, retrying attempt ${internalServerErrorAttempts}:`, result.statusText);
                                yield new Promise(resolve => setTimeout(resolve, internalServerErrorAttempts * 500));
                            }
                            else {
                                let payload = null;
                                try {
                                    payload = yield result.buffer();
                                }
                                catch (_a) {
                                    // relax lint
                                }
                                throw new Error(`TionApi - error ${result.status} ${result.statusText} ${payload === null || payload === void 0 ? void 0 : payload.toString()}`);
                            }
                        }
                        catch (err) {
                            if ((err.code === 'ESOCKETTIMEDOUT' || err.type === 'request-timeout') &&
                                internalServerErrorAttempts++ < 2) {
                                this.log.error(`TionApi - got timeout, retrying attempt ${internalServerErrorAttempts}:`, err.message);
                                yield new Promise(resolve => setTimeout(resolve, internalServerErrorAttempts * 500));
                            }
                            else {
                                this.wrapError(err);
                                throw err;
                            }
                        }
                        break;
                    case AuthState.NoToken:
                        this.log.debug('TionApi - no token');
                        internalServerErrorAttempts = 0;
                        try {
                            accessToken = yield this.authApi.authenticateUsingPassword();
                            state = AuthState.HasToken;
                        }
                        catch (err) {
                            this.log.error('TionApi - no_token:', err.message || err.statusCode);
                            this.wrapError(err);
                            throw err;
                        }
                        break;
                    case AuthState.TokenExpired:
                        this.log.debug('TionApi - token expired');
                        internalServerErrorAttempts = 0;
                        try {
                            accessToken = yield this.authApi.authenticateUsingRefreshToken();
                            state = AuthState.HasToken;
                        }
                        catch (err) {
                            this.log.error('TionApi - token_expired:', err.message || err.statusCode);
                            if (err.statusCode >= 400 && err.statusCode < 500) {
                                state = AuthState.NoToken;
                            }
                            else {
                                this.wrapError(err);
                                throw err;
                            }
                        }
                        break;
                }
            }
        });
    }
}
exports.TionApi = TionApi;
TionApi.ApiBasePath = 'https://api2.magicair.tion.ru';
//# sourceMappingURL=api.js.map